# Importing necessary modules
import serial
import sys
import time
from ipywidgets import interact, interactive, fixed, interact_manual
import roboticstoolbox as rtb
import numpy as np

class AL5DControl:
    def __init__(self):
        
        # al5d model
        self.al5d = rtb.models.URDF.AL5D_mdw()

        # search for port, initialise in home pose
        devices = ['/dev/ttyUSB0','/dev/ttyUSB1','COM1','COM2','COM3','COM4','COM5','COM6','COM7','COM8','COM9']
        
        # Starting the connection with the SSC-32U controller
        for device in devices:
            try:
                self.ssc32 = serial.Serial(device, 9600, timeout=1.0)
                # Initializing the position of the robot
                self.ssc32.write(b'#0P1500S100#1P1150S200#2P1500S300#3P1500S350#4P1500S100#5P1500S150\r')
            except serial.SerialException:
                print("No device connected on ", device)
        
    def robot_control(self, q0=0, q1=0, q2=0, q3=0, q4=0, gripper=0): #Method for controlling the joints
        q0r = q0*np.pi/180
        q1r = q1*np.pi/180
        q2r = q2*np.pi/180
        q3r = q3*np.pi/180
        q4r = q4*np.pi/180
        elbow = self.al5d.fkine([q0r,q1r,q2r,q3r,q4r], end='elbow')
        wrist = self.al5d.fkine([q0r,q1r,q2r,q3r,q4r], end='wrist')
        grip = self.al5d.fkine([q0r,q1r,q2r,q3r,q4r], end='gripper')
        tool = self.al5d.fkine([q0r,q1r,q2r,q3r,q4r], end='tool')

        if any(np.array([tool.t[2], grip.t[2], elbow.t[2], wrist.t[2]]) < 0.06):
            print("Robot collision with the table")
        else:
            print("No collision detected")
            pwq0 = str(round((q0+90)*11.1111 + 500)) # Scaling of the joint signal from (0,180) to (500,2500)
            pwq1 = str(round((q1+90)*11.1111 + 500))
            pwq2 = str(round((q2+90)*11.1111 + 500))
            pwq3 = str(round((q3+90)*11.1111 + 500))
            pwq4 = str(round((q4+90)*11.1111 + 500))
            pwq5 = str(round((gripper+90)*11.1111 + 500))
            # Concatinating the desired control signals in a long string
            output = "#0P" + pwq0 + "S300"+"#1P" + pwq1 + "S300"+"#2P" + pwq2 + "S300"+"#3P" + pwq3 + "S1500" + "#4P" + pwq4 + "S1500"+ "#5P" + pwq5 + "S1500"+"\r"  # Formatting string
            output = output.encode('utf-8')  # Converting to bytes literals
            self.ssc32.write(output)  # sending serial data to the SSC-32 board

    def send_commands(self):
        # Initialising the sliders
        interact(self.robot_control, q0=(-90,90), q1=(-90,90), q2=(-90,90), q3=(-90,90), q4=(-90,90), gripper=(-90,90))

